# -*- coding=utf-8 -*-
#Copyright 2012 Daniel Osvaldo Mondaca Seguel
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
import sys

reload(sys)
sys.setdefaultencoding('utf-8')


def getHeaders():
    headers = [("User-agent", "Feedfetcher-pyscrap")]
    return headers


def getPipes():
    pipes = {"spiders":
             {
             "demoSpider": "spiderDefaultSave",
             },
             "items":
             {
             "customSaveItem": "customSave"
             }
             }
    return pipes
